using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using SerialPorts;

namespace Serial_Communicator
{
	/// <summary>
	/// Terminal class uses SerialPorts.dll found at http://www.codeproject.com/dotnet/DotNetComPorts.asp
	/// to communicate through serial port with FPGA board.
	/// </summary>
	public class Terminal : System.Windows.Forms.Form
	{
		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;
		

		#region Members
		private System.Windows.Forms.MainMenu mainMenu1;
		
		// Terminal interface port.
		private SerialPort Port;
		private bool UseDefaultConfig = false;

		// Constants.
		private const int	MAX_PORTS  = 32;

		private System.Windows.Forms.TextBox rxTextBox;
		private System.Windows.Forms.MenuItem File;
		private System.Windows.Forms.MenuItem Exit;
		private System.Windows.Forms.Label txLabel;
		private System.Windows.Forms.Label rxLabel;
		private System.Windows.Forms.TextBox txTextBox;
		private System.Windows.Forms.Button SendTx;
		private System.Windows.Forms.MenuItem PortMenu;
		private System.Windows.Forms.Label StatusLabel;
		private System.Windows.Forms.TextBox Status;
		private System.Windows.Forms.MenuItem HelpMenu;
		private System.Windows.Forms.MenuItem AboutMenu;
		private System.Windows.Forms.MenuItem Configure;
		private System.Windows.Forms.MenuItem OpenPort;
		private System.Windows.Forms.MenuItem ClosePort;
		private System.Windows.Forms.Button rxCLR;
		private System.Windows.Forms.CheckBox txContinuous;

		// Terminal functions that 
		// handle base class events.
		private WithEvents	Func;
		#endregion
		#region Methods
		private void MakeMenu(MenuItem parentMenu)
		{
			OpenPortMenuItem m;
			string s;
			int index = 0;
			SerialPort p = new SerialPort(this.Func);
			for(int i = 1; i <= MAX_PORTS; i++)
			{
				this.Status.Text = "Checking for COM"+i.ToString()+"...";
				s = "COM" + i.ToString();
				if(p.Available(s))
				{
					this.Status.Text = "COM"+i.ToString()+" exists, adding to Port menu...";
					m = new OpenPortMenuItem(i,parentMenu);
					m.Text = s;
					parentMenu.MenuItems.Add(index,m);
					parentMenu.MenuItems[index].Click += new System.EventHandler(this.COM_Click);
					index++;
				}
			}
			return;
		}

		private void EnableComponents(bool enable)
		{
			this.txTextBox.Enabled = enable;
			this.SendTx.Enabled = enable;
			this.rxTextBox.Enabled = enable;
			this.rxCLR.Enabled = enable;
			this.txContinuous.Enabled = enable;
		}
		#endregion
		#region Constructor
		public Terminal()
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();

			// Fill Port > Open submenu with available ports
			this.Status.Text = "Checking for available COM ports...";
			this.MakeMenu(this.OpenPort);
			this.MakeMenu(this.Configure);

			// Instantiate base class event handlers.
			this.Func = new WithEvents();
			this.Func.Error   = new StrnFunc(this.OnError);
			this.Func.RxChar  = new ByteFunc(this.OnRecvI);
			this.Func.CtsSig  = new BoolFunc(this.OnCts);
			this.Func.DsrSig  = new BoolFunc(this.OnDsr);
			this.Func.RlsdSig = new BoolFunc(this.OnRlsd);
			this.Func.RingSig = new BoolFunc(this.OnRing);
			
			// Instantiate the terminal port.
			this.Port = new SerialPort(this.Func);
			this.Status.Text = "Program loaded: Configure and open port from menu above";
			this.Status.Select(0,0);
			this.Status.Focus();

			// Disable components
			EnableComponents(false);
			return;
		}
		#endregion
		#region Dispose
		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if (components != null) 
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}
		#endregion
		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(Terminal));
			this.mainMenu1 = new System.Windows.Forms.MainMenu();
			this.File = new System.Windows.Forms.MenuItem();
			this.Exit = new System.Windows.Forms.MenuItem();
			this.PortMenu = new System.Windows.Forms.MenuItem();
			this.Configure = new System.Windows.Forms.MenuItem();
			this.OpenPort = new System.Windows.Forms.MenuItem();
			this.ClosePort = new System.Windows.Forms.MenuItem();
			this.HelpMenu = new System.Windows.Forms.MenuItem();
			this.AboutMenu = new System.Windows.Forms.MenuItem();
			this.txLabel = new System.Windows.Forms.Label();
			this.rxLabel = new System.Windows.Forms.Label();
			this.txTextBox = new System.Windows.Forms.TextBox();
			this.rxTextBox = new System.Windows.Forms.TextBox();
			this.SendTx = new System.Windows.Forms.Button();
			this.StatusLabel = new System.Windows.Forms.Label();
			this.Status = new System.Windows.Forms.TextBox();
			this.rxCLR = new System.Windows.Forms.Button();
			this.txContinuous = new System.Windows.Forms.CheckBox();
			this.SuspendLayout();
			// 
			// mainMenu1
			// 
			this.mainMenu1.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																					  this.File,
																					  this.PortMenu,
																					  this.HelpMenu});
			// 
			// File
			// 
			this.File.Index = 0;
			this.File.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																				 this.Exit});
			this.File.Text = "File";
			// 
			// Exit
			// 
			this.Exit.Index = 0;
			this.Exit.Text = "Exit";
			this.Exit.Click += new System.EventHandler(this.Exit_Click);
			// 
			// PortMenu
			// 
			this.PortMenu.Index = 1;
			this.PortMenu.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																					 this.Configure,
																					 this.OpenPort,
																					 this.ClosePort});
			this.PortMenu.Text = "Port";
			// 
			// Configure
			// 
			this.Configure.Index = 0;
			this.Configure.Text = "Create Config File...";
			this.Configure.Click += new System.EventHandler(this.Configure_Click);
			// 
			// OpenPort
			// 
			this.OpenPort.Index = 1;
			this.OpenPort.Text = "Open";
			this.OpenPort.Click += new System.EventHandler(this.OpenPort_Click);
			// 
			// ClosePort
			// 
			this.ClosePort.Index = 2;
			this.ClosePort.Text = "Close";
			this.ClosePort.Click += new System.EventHandler(this.ClosePort_Click);
			// 
			// HelpMenu
			// 
			this.HelpMenu.Index = 2;
			this.HelpMenu.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																					 this.AboutMenu});
			this.HelpMenu.Text = "Help";
			// 
			// AboutMenu
			// 
			this.AboutMenu.Index = 0;
			this.AboutMenu.Text = "About Serial Communicator...";
			this.AboutMenu.Click += new System.EventHandler(this.AboutMenu_Click);
			// 
			// txLabel
			// 
			this.txLabel.Location = new System.Drawing.Point(8, 32);
			this.txLabel.Name = "txLabel";
			this.txLabel.Size = new System.Drawing.Size(64, 16);
			this.txLabel.TabIndex = 0;
			this.txLabel.Text = "Transmit";
			// 
			// rxLabel
			// 
			this.rxLabel.Location = new System.Drawing.Point(8, 160);
			this.rxLabel.Name = "rxLabel";
			this.rxLabel.Size = new System.Drawing.Size(64, 16);
			this.rxLabel.TabIndex = 1;
			this.rxLabel.Text = "Received";
			// 
			// txTextBox
			// 
			this.txTextBox.AutoSize = false;
			this.txTextBox.Enabled = false;
			this.txTextBox.Location = new System.Drawing.Point(48, 48);
			this.txTextBox.Multiline = true;
			this.txTextBox.Name = "txTextBox";
			this.txTextBox.ScrollBars = System.Windows.Forms.ScrollBars.Vertical;
			this.txTextBox.Size = new System.Drawing.Size(296, 88);
			this.txTextBox.TabIndex = 2;
			this.txTextBox.Text = "";
			this.txTextBox.TextChanged += new System.EventHandler(this.txTextBox_TextChanged);
			// 
			// rxTextBox
			// 
			this.rxTextBox.AutoSize = false;
			this.rxTextBox.Location = new System.Drawing.Point(48, 176);
			this.rxTextBox.Multiline = true;
			this.rxTextBox.Name = "rxTextBox";
			this.rxTextBox.ReadOnly = true;
			this.rxTextBox.ScrollBars = System.Windows.Forms.ScrollBars.Vertical;
			this.rxTextBox.Size = new System.Drawing.Size(296, 88);
			this.rxTextBox.TabIndex = 3;
			this.rxTextBox.Text = "";
			// 
			// SendTx
			// 
			this.SendTx.Enabled = false;
			this.SendTx.Location = new System.Drawing.Point(160, 144);
			this.SendTx.Name = "SendTx";
			this.SendTx.Size = new System.Drawing.Size(64, 24);
			this.SendTx.TabIndex = 4;
			this.SendTx.Text = "Send";
			this.SendTx.Click += new System.EventHandler(this.SendTx_Click);
			// 
			// StatusLabel
			// 
			this.StatusLabel.Location = new System.Drawing.Point(8, 8);
			this.StatusLabel.Name = "StatusLabel";
			this.StatusLabel.Size = new System.Drawing.Size(64, 16);
			this.StatusLabel.TabIndex = 5;
			this.StatusLabel.Text = "Status";
			// 
			// Status
			// 
			this.Status.BackColor = System.Drawing.SystemColors.Info;
			this.Status.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
			this.Status.Enabled = false;
			this.Status.Location = new System.Drawing.Point(48, 6);
			this.Status.Name = "Status";
			this.Status.ReadOnly = true;
			this.Status.Size = new System.Drawing.Size(312, 20);
			this.Status.TabIndex = 6;
			this.Status.Text = "";
			this.Status.TextAlign = System.Windows.Forms.HorizontalAlignment.Center;
			// 
			// rxCLR
			// 
			this.rxCLR.Location = new System.Drawing.Point(144, 272);
			this.rxCLR.Name = "rxCLR";
			this.rxCLR.Size = new System.Drawing.Size(96, 24);
			this.rxCLR.TabIndex = 7;
			this.rxCLR.Text = "Clear Received";
			this.rxCLR.Click += new System.EventHandler(this.rxCLR_Click);
			// 
			// txContinuous
			// 
			this.txContinuous.Location = new System.Drawing.Point(240, 144);
			this.txContinuous.Name = "txContinuous";
			this.txContinuous.Size = new System.Drawing.Size(136, 24);
			this.txContinuous.TabIndex = 8;
			this.txContinuous.Text = "Continuously Transmit";
			this.txContinuous.Click += new System.EventHandler(this.txContinuous_Click);
			// 
			// Terminal
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(384, 297);
			this.Controls.Add(this.txContinuous);
			this.Controls.Add(this.rxCLR);
			this.Controls.Add(this.Status);
			this.Controls.Add(this.rxTextBox);
			this.Controls.Add(this.txTextBox);
			this.Controls.Add(this.StatusLabel);
			this.Controls.Add(this.SendTx);
			this.Controls.Add(this.rxLabel);
			this.Controls.Add(this.txLabel);
			this.Icon = ((System.Drawing.Icon)(resources.GetObject("$this.Icon")));
			this.MaximizeBox = false;
			this.Menu = this.mainMenu1;
			this.Name = "Terminal";
			this.Text = "Serial Communicator";
			this.Closing += new System.ComponentModel.CancelEventHandler(this.Terminal_Closing);
			this.ResumeLayout(false);

		}
		#endregion
		#region Main
		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main() 
		{
			Application.Run(new Terminal());
		}
		#endregion
		#region Hooks
		/// <summary>
		/// Handles error events.
		/// </summary>
		internal void OnError(string fault)
		{
			// Do Something Here //
		}

		/// <summary>
		/// Set the modem state displays.
		/// </summary>
		internal void OnCts(bool cts)
		{
			// Do Something Here //
		}

		/// <summary>
		/// Set the modem state displays.
		/// </summary>
		internal void OnDsr(bool dsr)
		{
			// Do Something Here //
		}

		/// <summary>
		/// Set the modem state displays.
		/// </summary>
		internal void OnRlsd(bool rlsd)
		{
			// Do Something Here //
		}

		/// <summary>
		/// Set the modem state displays.
		/// </summary>
		internal void OnRing(bool ring)
		{
			// Do Something Here //
		}

		/// <summary>
		/// Immediate byte received.
		/// </summary>
		internal void OnRecvI(byte[] b)
		{
			this.rxTextBox.Text = this.rxTextBox.Text + System.Text.Encoding.ASCII.GetString(b);
		}
		#endregion
		#region EventMethods
		private void Exit_Click(object sender, System.EventArgs e)
		{
			this.Port.Close();
			this.Dispose();
		}

		private void OpenPort_Click(object sender, System.EventArgs e)
		{
			
		}

		private void ClosePort_Click(object sender, System.EventArgs e)
		{
			string name = this.Port.Cnfg.PortName;
			this.Status.Text = "Closing Port "+name+"...";
			this.Port.Close();
			this.Status.Text = "Port "+name+" closed.";
			this.EnableComponents(false);
		}

		private void AboutMenu_Click(object sender, System.EventArgs e)
		{
			MessageBox.Show("Serial Communicator v0.1.1\n\nThis program was written to communicate through the serial port\nusing as simple an interface as possible.\n\nCreated by Jae Sammy on 07/12/2004", "About Serial Communicator");
		}

		private void Configure_Click(object sender, System.EventArgs e)
		{
			
		}

		private void COM_Click(object sender, System.EventArgs e)
		{
			OpenPortEventArgs a = e as OpenPortEventArgs;
			if (a.Parent == this.OpenPort)
			{
				if(a != null)
				{
					if(this.Port.IsOpen) // Port already opened, switch ports
					{
						string name = this.Port.Cnfg.PortName;
						this.Status.Text = "Closing Port "+name;
						this.Port.Close();
						if( this.Port.Open(a.PortNum) == false)
						{
							this.Status.Text = "Port COM"+a.PortNum+" failed to open!  Have you configured the port?";
						}
						else
						{
							this.Status.Text = "Port "+name+" closed and Port COM"+a.PortNum+" opened.";
							EnableComponents(true);
							this.txContinuous.Checked = true;
							this.SendTx.Enabled = false;
						}
					}
					else // Port closed, try to config and open port
					{
						if( this.Port.Open(a.PortNum) == false)
						{
							this.Status.Text = "Port COM"+a.PortNum+" failed to open!  Have you configured the port?";
						}
						else
						{
							this.Status.Text = "Port COM"+a.PortNum+" opened.";
							EnableComponents(true);
							this.txContinuous.Checked = true;
							this.SendTx.Enabled = false;
						}
					}
				}
			} 
			else if (a.Parent == this.Configure) 
			{
				Configure form = new Configure("Port"+a.PortNum.ToString()+".cfg");
				form.ShowDialog();
			}
		}
		private void SendTx_Click(object sender, System.EventArgs e)
		{
			for (int i = 0; i< this.txTextBox.Text.Length;i++)
				this.Port.SendI(Convert.ToByte(this.txTextBox.Text[i]));
		}

		private void Terminal_Closing(object sender, System.ComponentModel.CancelEventArgs e)
		{
			this.Port.Close();
		}

		private void rxCLR_Click(object sender, System.EventArgs e)
		{
			this.rxTextBox.Text = "";
		}

		private void txContinuous_Click(object sender, System.EventArgs e)
		{
				this.SendTx.Enabled = !this.txContinuous.Checked;
		}

		private void txTextBox_TextChanged(object sender, System.EventArgs e)
		{
			if (this.txContinuous.Checked && (this.txTextBox.TextLength>0))
				this.Port.SendI(Convert.ToByte(this.txTextBox.Text[this.txTextBox.TextLength-1]));
		}
		#endregion

	}
}
